#! /usr/local/bin/perl
#
# This is a perl version of unix file
# To add test to a type of file, just put a sub at the
# end of this program which test the file type.
# 
# It reads the first 512 bytes into $head, change all '\n' to ' '
# and put the result to $HEAD. Then you can do regexp match on any
# one of these two strings.
#
# creator: yufan@cs.keele.ac.uk (Yufan Hu)
#

open(PROG, $0) || die "$0 cannot open\n";
while(<PROG>){
	if(/sub\s+(\w+)\s*{/){
		push(@magic, $1);
	}
}
close PROG;

$|=1;
for $f (@ARGV){
	$fstr = sprintf("%-15s\t", "$f:");
	if(! -r $f){
		print $fstr, "cannot open\n"; next;
	}
	undef $head; undef $HEAD;
	open(FILE, $f);
	$r = read(FILE, $head, 512); close(FILE);
	if(!$r){ push(@unread, $f); next;}
	$yes = 0;
	$HEAD = $head;
	$HEAD =~ s/\n/ /g;
	@head = unpack("c20", $head);
	for $try (@magic){
		if( eval "&$try" ){
			$yes = 1; last;
		}
	}
	if(!$yes){ push(@unread, $f);}
}

if(@unread){ system "/usr/bin/file @unread"; }

sub isgif {
        if($head =~ /^GIF8[79]a/){print $fstr, unpack("aaaaa",$head), " image file\n";}
}

sub isjpeg {
	if($head =~ /^......JFIF/){print $fstr, "JPEG compressed image file\n";}
}

sub isppm {
	if($head =~ /^(P[1-6])\s+/){
		local($type);
		if($1 eq 'P1'){$type = 'PBM';}
		if($1 eq 'P2'){$type = 'PGM';}
		if($1 eq 'P3'){$type = 'PPM';}
		if($1 eq 'P4'){$type = 'PBM "rawbits"';}
		if($1 eq 'P5'){$type = 'PGM "rawbits"';}
		if($1 eq 'P6'){$type = 'PPM "rawbits"';}
		print $fstr, "PPM $type image file\n";
	}
	elsif($head =~ /^P6\s/){
		printf $fstr, "Raw PPM image file\n";
	}
}

sub iscmd {
	if(-x $f && $head =~ /^#!([\w\/-]*)/){
		print $fstr,"executable $1 commands text\n";
	}
}

sub iscompressed {
	if($head =~ /^\037\235/){print $fstr, "compressed file\n";}
}

sub isuudecode {
	if($head =~ /^begin/){ print $fstr, "uuencoded file\n";}
}

sub isbtoa {
	if($head =~ /^xbtoa/){print $fstr, "btoa'd file\n";}
}

sub isSunrasterfile {
	if($head =~ /^\131\246\152\225/){print $fstr, "Sun rasterfile\n";}
}

sub tiff {
	if($head =~ /^\115\115/){
		print $fstr, "TIFF file, big-endian\n";
	} elsif($head =~ /^\111\111/){
		print $fstr, "TIFF file, little-endian\n";
	}
}

sub ImageMagick {
	if($HEAD =~ /^\s*\{\s*(Imp.*)\}\s*id=ImageMagick/){
		print $fstr, "ImageMagic MIFF image file\n";
	}
}

sub PostScript {
	if($head =~ /^\%!(PS-Adobe)*|TeXDict begin|\/showpage|\/\w+.*\sdef/){
		local($creator) = $HEAD =~ /Creator:\s*([^%]*)\s+%%/;
		print $fstr, "PostScript file (Created by $creator)\n";
	}
}

sub xbibmap {
	if($HEAD =~ /^\s*#define.*width\s+(\d+)\s*#define\s.*height\s+(\d+).*static\s+char.*\[\]\s*=\s*{/){
		print $fstr, "xbitmap file\n";
	}
}

sub xfig {
	if($head =~ /^#FIG\s+([\d.]+)/){print $fstr, "xfig $1 data file\n";}
}

sub cplus {
	if($HEAD =~ /\s*#ifndef.*#define/){
		print $fstr, "C/C++ Header file\n";
	} elsif($HEAD =~ /class\s[\w:,\s]+{|[\w]+\s*::\s*\w*\s*\(.*[{;]|public:|provate:|#include\s*\<stream.h\>/){
		print $fstr, "C++ file\n";
	}
}

sub iv_doc {
	if($head =~ /^%(Doc-\d+\.\d+)/){print $fstr, "InterViews $1 file\n";}
}

sub ibuild {
	if($head =~ /^%I Unidraw/){ print $fstr, "InterViews ibuild data file\n";}
}

sub latex {
	if($HEAD =~ /\\documentstyle|\\section{|\\chapter{\\subsection{|\\subsubsect{/||
	   $HEAD =~ /\\newpage|\\doublespace|\\center/){
		print $fstr, "LaTex file\n";
	}elsif($head =~ /^\\relax/){
		print $fstr, "TeX AUX file\n";
	} elsif($head =~ /^\\contentsline\s+{/){
		print $fstr, "TeX TOC file\n";
	} elsif($head =~ /^This is TeX,/){
		print $fstr, "TeX LOG file\n";
	} elsif(ord($head) == 247){
		print $fstr, "DVI file\n";
	}
}

sub mail_news {
	if($HEAD =~ /From:.*To:/){print $fstr, "Email \n";}
	elsif($HEAD =~ /From:.*Newsgroups:\s*([^:]+)\s[A-Z]/){ print $fstr, "Usenet news article ($1)\n";}
}

sub ispic {
	if($head =~ /^\x34\x12/){ print $fstr, "PIC image file\n";}
}

