# soundex.pl
# by George Armhold <armhold@dimacs.rutgers.edu> 3/22/92
# improvements by Marc Arnold <marc@mit.edu>

# return the Soundex value of a string using the following rules:
#
#   1) remove W and H
#   2) remove all vowels except in the first position (A E I O U Y)
#   3) recode characters per table:
#           A E I O U Y             0
#           B F P V                 1
#           C G J K Q S X Z         2
#           D T                     3
#           L                       4
#           M N                     5
#           R                       6
#
#   4) if two adjacent digits are now identical, remove one
#   5) truncate to six digits or pad out the result with zeroes to
#   make six digits  
#   6) replace the first digit with the first character from the
#   original word 

sub soundex {
# takes a string as an argument, and returns its soundex value

    local($pattern) = @_;

    # upper-case the pattern to normalize matches
    $pattern =~ tr/a-z/A-Z/;

    # remove all but alphanumerics, and H,W
    $pattern =~ tr/A-GI-VX-Z0-9//cd;

    # save first char
    local($first) = substr($pattern, 0, 1);

    # remove all vowels after 1st letter
    substr($pattern, 1, length($pattern)) =~ tr/AEIOUY//d;
   
    # replaces letters with numbers and squish identical numbers
    $pattern =~ tr/BFPVCGJKQSXZDTLMNR0-9/1111222222223345560-9/ds;

    # replace first letter
    substr($pattern, 0, 1) = $first;

    # pad on zeroes if necessary and truncate
    substr($pattern."000000", 0, 6); 
}

1;				# because this is a require'd file

--
-----------------------------------------------------------------------
Lucien Van Elsen                                          IBM  Research
lucien@watson.ibm.com                                     Project Agora

From: stetzer@csd4.csd.uwm.edu (Frank Stetzer)
Subject: Re: SOUNDEX pattern matching
Date: 20 Jan 1993 16:30:01 GMT
Organization: Computing Services Division, University of Wisconsin - Milwaukee
Lines: 57

A recent re-post of a soundex routine did not include some
of the follow-up corrections.  Here they are.

-----------------------followup #1------------------------------------

~From: armhold@dimacs.rutgers.edu (George Armhold)
~Newsgroups: comp.lang.perl
~Subject: Re: improved soundex.pl
~Date: 24 Mar 92 19:10:01 GMT

Another fix:  replace the first char code with the ORIGINAL char,
before W, H & vowels are removed.  Here's a diff:

33,35d32
<     # save first char
<     local($first) = substr($pattern, 0, 1);
< 
37a35,37
> 
>     # save first char
>     local($first) = substr($pattern, 0, 1);



-George
-- 
		 Internet: armhold@dimacs.rutgers.edu
	       UUCP: {backbone}!rutgers!dimacs!armhold
		    BITNET: armhold@PISCES.BITNET

---------------------Followup #2-----------------------------------
~Newsgroups: comp.lang.perl
~From: tye@spillman.uucp (E. Tye McQueen)
~Subject: Re: improved soundex.pl
~Date: Thu, 26 Mar 1992 18:11:21 GMT

Marc Horowitz <marc@mit.edu> writes:
># soundex.pl
[...]
>#   2) remove all vowels except in the first position (A E I O U Y)
[...]
>#   4) if two adjacent digits are now identical, remove one

Remove vowels *after* you remove redundant digits or "banana"
sounds the same as "ban".

 Tye McQueen, E.     [if uucp]             [if internet]
                   [...uunet!]spillman!tye[@uunet.uu.net]
----------------------------------------------------------
 Nothing is obvious unless you are overlooking something. 
----------------------------------------------------------
----------------End of Followups--------------------------------------
==========================================================================
Frank Stetzer                                   stetzer@convex.csd.uwm.edu
Statistical Software Coordinator                P.O. Box 413
Computing Services Division                     Milwaukee, WI 53201
University of Wisconsin - Milwaukee             (414)229-4571

