;# NAME
;#    timeout.pl - execute Perl code with a built-in timeout
;#
;# AUTHOR
;#    Michael S. Muegel (mmuegel@mot.com)
;#
;# RCS INFORMATION
;#    $Author: mmuegel $
;#    $Header: /usr/local/lib/perl/RCS/timeout.pl,v 1.1 1992/05/29 03:07:35 mmuegel Exp $

package Timeout;

# Messages sent to die from eval for alarm call
$TIMEOUT = "timeout";

;###############################################################################
;# Timeout
;#
;# Executes the Perl code in $Code with a built-in timeout of $Timeout
;# seconds. This is very useful when parsing input from a user, socket,
;# or file and you wish to fix a timeout on your parse.
;#
;# $Code is executed with an eval. Thus you can use die to break out of
;# it prematurely. $Code is also executed in the package $Package. Thus your
;# Perl code can have access to other variables within its package. This also
;# means you can set variables in $Code and they will be available in the
;# package $Package. If $Package is not specified main is assumed.
;#
;# $Status is set to one of the following upon return:
;#
;#    -1	Eval of $Code failed. This can be because of a Perl compiler 
;#		error or a die inside the code was used. See $Msg for reason 
;#		why.
;#
;#     0	A timeout occured.
;#
;#     1	No timeout occured.
;#
;# Arguments:
;#    $Timeout, $Code, $Package
;#
;# Returns:
;#    $Status, $Msg
;###############################################################################
sub main'Timeout
{
   ($Timeout, $Code, $Package) = @_;

   # The main package is assumed
   $Package = "main" if ($Package eq "");

   # Save the interrupt handler vector and then override the handler for ALRM
   %Save_SIG = %SIG;
   $SIG {'ALRM'} = "Timeout'Alarm_Handler";

   # Set up a timer for $Timeout seconds
   alarm ($Timeout);
   
   # Do our thing here within eval in the $Package package. This way we can die 
   # from the interrupt handler.
   eval "package $Package; $Timeout'Code";

   # Restore signal handlers
   %SIG = %Save_SIG;

   # Check eval status
   alarm (0);
   return (0) if ($@ =~ /^$TIMEOUT/);
   return (-1, $@) if ($@);
   return (1);

};

sub Alarm_Handler
{
   die $TIMEOUT if ($_[0] eq "ALRM");
};

1;
