#! /usr/local/bin/perl
#
#	$Id: uucombine,v 1.5 91/08/13 06:59:53 doi Exp $
#
#	uucombine - a front-end to uudecode to combine the uuencoded
#	and split files; then uudecode it
#
#	original hack by Hitoshi Doi (doi@decvax.dec.com), 1991/01/15
#	this program is public domain.  do whatever you wish with it.
#
#	---------------------------------------------------------------
#	modifications
#
#	1991/03/11	Hitoshi Doi
#		handle files that have already been concatenated.
#
#	1991/06/26	Hitoshi Doi
#		added the -outdir option.  added checks for the tmpdir.
#
#	1991/08/12	Hitoshi Doi
#		added the -name option.
#	---------------------------------------------------------------

	$me = $0;
	$me =~ s/.*\///;

	($tmpdir = $ENV{'TMP'}) || ($tmpdir = '/tmp');
	if (!-d $tmpdir) {
		$tmpdir = '.';
	}
	$tfile = "$tmpdir/uuc.$$";
	open(T, ">$tfile");

	$verbose = 0;		# verbose is off
	$odir = '.';		# put output in current directory
	$oname = "";		# name of ouput file

	$i = 0;
	$first = 1;
	files_loop: while ($ifile = $ARGV[$i]) {
		if ($ifile eq "-verbose") {
			$verbose = 1;
			$i++;
			next files_loop;
		}
		elsif ($ifile eq "-outdir") {
			$i++;
			$odir = $ARGV[$i];
			$i++;
			next files_loop;
		}
		elsif ($ifile eq "-name") {
			$i++;
			$oname = $ARGV[$i];
			$i++;
			next files_loop;
		}
		elsif ($ifile eq "-help") {
			print "
$me - combine the uuencoded and split files,
      then uudecode it in the current or specified directory.

usage: $me [options] files...
options: -verbose      give lots of details
         -outdir dir   put the uudecoded file in directory dir
         -name file    specify the name of the uudecoded file
";
			close(T);
			unlink($tfile);
			exit 0;
		}
		#
		# current input file
		#
		if (!open(I, "<$ifile")) {
			print "$me: can't open $ifile, skipping it\n";
			$i++;
			next files_loop;
		}
		@in = <I>;
		close(I);
		$j = 0;
		$uuin = 0;
		$uuout = 0;
		line_loop: foreach $l (@in) {
			$j++;
			if ($uuout) {
				if ($l =~ /^M.{60}/) {
					print T $l;
					$uuin = 1;
					$uuout = 0;
				}
				elsif ($verbose) {
					print "throwing away: $l";
				}
				next line_loop;
			}
			elsif ($uuin) {
				if ($l =~ /^M.{60}/) {
					print T $l;
				}
				else {
					$uuout = $j;
				}
				next line_loop;
			}
			#
			# special processing for the first file
			#
			if ($first) {
				if ($l =~ /^begin[ \t]+[0-9]+/i) {
					if (!$oname) {
						chop $l;
						@f = split(/[ \t]+/, $l);
						$f[2] =~ s/.*\///;
						$f[2] =~ y/A-Z/a-z/;
						$f[2] =~ s/(.*)\.z$/$1.Z/;
						$oname = $f[2];
					}
					print T "begin 640 $odir/$oname\n";
					$first = 0;
					$uuin = 1;
				}
				elsif ($verbose) {
					print "throwing away: $l";
				}
			}
			else {
				if ($l =~ /^M.{60}/) {
					print T $l;
					$uuin = 1;
				}
				elsif ($verbose) {
					print "throwing away: $l";
				}
			}
		} # end of line_loop
		$i++;
	} # end of files_loop
	#
	# special processing for the last file
	#
	$j = $uuout - 1;
	while ($l = $in[$j]) {
		print T $l;
		if ($l =~ /^end/) {
			last;
		}
		$j++;
	}
	close(T);

	system "uudecode $tfile";
#	unlink($tfile);
	exit 0;



